/* tetris.c:
	A tetris game with slime.
	Requires Allegro 3.9.32 or better.
	See readme.txt for more info.
*/
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <allegro.h>

#define _ int
#define a void
#define n(a,b) for(a=0; a<b; a++)
#define o return
#define p while
#define u BITMAP *
#define D SCREEN_W
#define E SCREEN_H
#define J ) QUIT("Out of memory");

/* Layout: board[y][x] is the yth box from top and the xth from left. */
_ c[20][10], e, f, h, C,
	 d, v, z, j, k,
	 m[8],
	 A, B, t,
/* Low 16 bits are for the shape of the piece.
*/
	 Go[7][4] =
{
	/* 1-right 2-down 4-left 8-up */
	/*    #                 ##
			#         #        #      ###
			##      ###        #      #
	*/
	{ 0x62260, 0x90470, 0x66440, 0x90710 },
	/*     #                ##
			 #      ###       #       #
			##        #       #       ###
	*/
	{ 0x64460, 0x90740, 0x66220, 0x90170 },
	/*    #                 #
			#       ####      #       ####
			#                 #
			#                 #
	*/
	{ 0x92222, 0x60f00, 0x92222, 0x60f00 },
	/*              #                #
			#        ##      ###       ##
		  ###        #       #        #
	*/
	{ 0x90270, 0x64640, 0x90720, 0x62620 },
	/*              #                 #
		  ##        ##      ##        ##
			##       #        ##       #
	*/
	{ 0x90360, 0x64620, 0x90360, 0x64620 },
	/*             #                 #
			##       ##       ##       ##
		  ##         #      ##         #
	*/
	{ 0x90630, 0x62640, 0x90630, 0x62640 },
	/*
			##       ##       ##       ##
			##       ##       ##       ##
	*/
	{ 0x00660, 0x00660, 0x00660, 0x00660 },
};



volatile _ s = 0;
u q, *F, *Gp, *G[56],
		 *score_bmp;
PALETTE Ga;
struct
{
	_ d, b, x, y, dx, dy, fx, fy, c;
} l[20+1][10+1];

#define Gb(piece, x, y) ((piece >> (x + 12-y*4)) & 1)



a Gc()
{
	s += 10;
}
END_OF_FUNCTION(Gc);



/* rel_rectfill:
	Helper for doing rectfill with specified width and height rather than x2
	and y2.
*/
a Gd(u i, _ x, _ y, _ w, _ g, _ b)
{
	rectfill(i, x, y, x + w - 1, y + g - 1, b);
}



/* Ge:
	Clamps the given color to the range 0..255.
*/
Ge(_ x)
{
	o  x & ~255 ?
					x < 0 ?
						0
					:
						255
				:
					x;
}



Gf(u bmp)
{
	_ x, y, i, r, r_, ra = 0,
		 rb[256], rc, rd, re, b, rf;
	unsigned char **g, *rg, *rh, *ri;
	unsigned long rj[256], rk;

	/* Precalculate random numbers. */
	n(i,256)
		rj[i] = rand();
	rk = rj[0];
	#define RAND() (ra = (ra + 1) & 255, rj[ra] += rk, rk = rj[ra])

	g = bmp->line;
	for(y = 0; y < 512; y += 64)
		for(x = 0; x < 512; x += 64)
			g[y][x] = RAND() & 255;

	r = 128;
	rc = 512 << 16;
	re = fsqrt(0x8000);
	for(i = 128; i > 1; i /= 2){
		r /= 2;

		rc = fmul(rc, 0x8000);
		rf = -128 * rc + rc / 2;
		n(b,256)
			rb[b] = rf >> 24,
			rf += rc;

		/* Centres of squares. */
		for(y = r; y < 512; y += i) {
			rg = g[y - r] - r;
			ri = g[y];
			rh = g[(y + r) & 0x1ff] - r;
			r_ = 512 + r;
			for(x = r; x < r_; x += i)
				ri[x] =
				 Ge(
				  (((_)rg[x] +
					 (_)rg[(x + i) & 0x1ff] +
					 (_)rh[x] +
					 (_)rh[(x + i) & 0x1ff] +
					 2) >> 2) + rb[RAND() & 255]);
		}

		rd = fmul(rc, re);
		rf = -128 * rd + rd / 2;
		n(b,256)
			rb[b] = rf >> 24,
			rf += rd;

		/* Centres of diamonds. */
		ri = g[512 - r];
		rh = g[0];
		for(y = 0; y < 512; y += r) {
			rg = ri;
			ri = rh;
			rh = g[(y + r) & 0x1ff];
			for(x = (y + r) & r; x < 512; x += i)
				ri[x] =
				 Ge(
				  (((_)rg[x] +
					 (_)ri[(x - r) & 0x1ff] +
					 (_)ri[(x + r) & 0x1ff] +
					 (_)rh[x] +
					 2) >> 2) + rb[RAND() & 255]);
		}
	}
	n(y,512*512)
		g[0][y] /= 4;

	o 0;
}



/* Gg:
*/
a Gg(u i, _ b, u rf)
{
	unsigned char **r, **g;
	_ x, y;

	r = rf->line;
	g = i->line;
	b = 64 + 20 * b;
	n(x,e*f)
		r[0][x] = rand();
	#define K(xx,yy)   (_)r[(y+yy)%f][(x+xx)%e]
	#define KLINE(yy)  (K(yy,0)+2*K(yy,1)+2*K(yy,2)+2*K(yy,3)+K(yy,4))
	n(y,f)
		n(x,e)
			g[y][x] = b + (K(0,1)+K(0,2)+K(0,3)+
											KLINE(1)+KLINE(2)+KLINE(3)+K(2,2)*2+
											K(4,1)+K(4,2)+K(4,3)) / 300;

	n(x,e)
		g[0][x] = b + 4 + (g[0][x]-b) * 0.8,
		g[f-1][x] = b + (g[f-1][x]-b) * 0.8;

	n(y,f)
		g[y][0] = b + 4 + (g[y][0]-b) * 0.8,
		g[y][e-1] = b + (g[y][e-1]-b) * 0.8;
}



/* Gh:
*/
a Gh()
{
	_ x, y, xx, yy, g, dx, dy, fx, fy, r, r_, i;

	n(y,20)
		for(x = 1; x < 10; x++)
			(c[y][x] | c[y][x-1]) ?
				dx = l[y+1][x].x - l[y][x].x,
				dy = l[y+1][x].y - l[y][x].y,
				g = fhypot(dx, dy),
				r = g - (f << 16),

				r_ = fdiv(r, g) >> 6,
				fx = fmul(r_, dx),
				fy = fmul(r_, dy),
				l[y][x].fx += fx,
				l[y][x].fy += fy,
				l[y+1][x].fx -= fx,
				l[y+1][x].fy -= fy : 0;

	for(y = 1; y < 20; y++)
		n(x,10)
			(c[y][x] | c[y-1][x]) ?
				dx = l[y][x+1].x - l[y][x].x,
				dy = l[y][x+1].y - l[y][x].y,
				g = fhypot(dx, dy),
				r = g - (e << 16),

				r_ = fdiv(r, g) >> 6,
				fx = fmul(r_, dx),
				fy = fmul(r_, dy),
				l[y][x].fx += fx,
				l[y][x].fy += fy,
				l[y][x+1].fx -= fx,
				l[y][x+1].fy -= fy : 0;

	if(!t)
		n(y,4)
			n(x,4)
				if(Gb(d, x, y))
					for(yy = k+y-1; yy <= k+y+2; yy++)
						for(xx = j+x-1; xx <= j+x+2; xx++)
							(xx >= 0) && (xx <= 10) &&
							(yy >= 0) && (yy <= 20) ?
								l[yy][xx].fx += (e << 16) / 80 / 6 *
																 SGN(j+x-xx),
								l[yy][xx].fy += (f << 16) / 80 / 6 *
																 SGN(k+y-yy) : 0;

	for(y = 1; y < 20; y++)
		for(x = 1; x < 10; x++)
			l[y][x].c ?
				dx = l[y][x].x - l[y][x].d,
				dy = l[y][x].y - l[y][x].b,

				fx = dx * 0.25/16,
				fy = dy * 0.25/16,
				l[y][x].fx -= fx,
				l[y][x].fy -= fy : 0;

	/* Update position. */
	i = 0;
	for(y = 1; y < 20; y++)
		for(x = 1; x < 10; x++) {
			l[y][x].c ?
				l[y][x].dx += l[y][x].fx,
				l[y][x].dy += l[y][x].fy,
				l[y][x].x += l[y][x].dx,
				l[y][x].y += l[y][x].dy : 0;

			l[y][x].y > i ?
				i = l[y][x].y : 0;
		}

	n(y,20+1)
		n(x,10+1)
			l[y][x].fx = fmul(l[y][x].dx, -0.05 * 65536),
			l[y][x].fy = fmul(l[y][x].dy, -0.05 * 65536);
}



/* Gi:
	Removes all lines that are completed, and returns the score addition.
*/
Gi()
{
	_ g, x, y, r, b = 0;

	n(y,20) {
		g = 0;
		n(x,10)
			if(!c[y][x]) {
				g = 1;
				break;
			}
		if(!g) {
			b++;
			memmove(c[1], c[0],
					  y * 10 * sizeof(_));
			memset(c[0], 0, 10 * sizeof(_));
			for(r = y-1; r >= 0; r--)
				for(x = 1; x < 10; x++)
					l[r + 1][x].x = l[r][x].x,
					l[r + 1][x].y = l[r][x].y + (f << 16),
					l[r + 1][x].dx = l[r][x].dx,
					l[r + 1][x].dy = l[r][x].dy,
					l[r + 1][x].fx = l[r][x].fx,
					l[r + 1][x].fy = l[r][x].fy,
					l[r + 1][x].c = l[r][x].c;

			n(x,10+1)
				l[0][x].c = 0;
		}
	}

	A += b;
	o b ? (2 << b) : 1;
}



/* Gj:
*/
a Gj(u rz, u bmp)
{
	_ x, y, r, g,
		 r_, ra, rb, rc, dx, dy, ddx, ddy,
		 b, i, rd, re,
		 rf, rg = 0;
	unsigned char *rh, **ri, **rj;

	/* Draw c. */
	n(y,20)
		n(x,10)
			c[y][x] ?
				blit(G[c[y][x] - 1], rz,
					  0, 0, (x+1)*e, (y+1)*f, e, f)
			:
				Gd(rz, (x+1)*e, (y+1)*f,
								 e, f, 0);

	/* Distort from rz to bmp. */
	ri = bmp->line;
	rj = rz->line;
	n(g,20) {
		rf = 0;
		n(r,10) {
			r_ = l[g][r].x;
			ra = l[g][r].y;
			dx = (l[g][r+1].x -
					l[g][r].x) / e;
			dy = (l[g][r+1].y -
					l[g][r].y) / e;
			rb = (l[g+1][r].x -
							l[g][r].x) / f;
			rc = (l[g+1][r].y -
							l[g][r].y) / f;
			rd = (l[g+1][r+1].x -
							 l[g+1][r].x) / e;
			re = (l[g+1][r+1].y -
							 l[g+1][r].y) / e;
			ddx = (rd - dx) / f;
			ddy = (re - dy) / f;
			n(y,f) {
				rh = ri[y] + rf;
				b = r_;
				i = ra;
				n(x,e)
					rh[x] = rj[i >> 16][b >> 16],
					b += dx,
					i += dy;

				r_ += rb;
				ra += rc;
				dx += ddx;
				dy += ddy;
			}
			rf += e;
		}
		ri += f;
	}

	/* Draw piece. */
	if(!t)
		n(y,4)
			n(x,4)
				if(Gb(d, x, y))
					blit(G[m[rg++]], bmp,
						  0, 0, (j+x) * e, (k+y) * f,
						  e, f);
}



/* H:
	Returns 1 if it is ok to put the piece at (x,y).
*/
H(_ piece, _ x, _ y)
{
	_ r, g;

	n(g,4)
		n(r,4)
			if(Gb(piece, r, g)) {
				if((x+r >= 10) | (y+g >= 20) |
					(x+r < 0) | (y+g < 0))
					o 0;
				if(c[y+g][x+r])
					o 0;
			}

	o 1;
}



/* G_:
*/
a G_(_ x, _ y)
{
	l[y][x].c = 1;
	l[y][x].x = l[y][x].d;
	l[y][x].y = l[y][x].b;
	l[y][x].dx = l[y][x].dy = l[y][x].fx = l[y][x].fy = 0;
}



/* I:
	Puts the piece at the specified position, returning the change in score.
*/
I(_ piece, _ x, _ y)
{
	_ r, g, b = 0;

	n(g,4)
		n(r,4)
			if(Gb(piece, r, g))
				c[y+g][x+r] = 1 + m[b++],

				G_(x+r, y+g),
				G_(x+r, y+g+1),
				G_(x+r+1, y+g),
				G_(x+r+1, y+g+1),

				l[y+g][x+r].fy += (f << 16) / 45,
				l[y+g][x+r].fx -= (e << 16) / 45,
				l[y+g][x+r+1].fy += (f << 16) / 45,
				l[y+g][x+r+1].fx += (e << 16) / 45,
				l[y+g+1][x+r].fy += (f << 16) / 45,
				l[y+g+1][x+r].fx -= (e << 16) / 45,
				l[y+g+1][x+r+1].fy += (f << 16) / 45,
				l[y+g+1][x+r+1].fx += (e << 16) / 45;

	o Gi();
}



Gk()
{
	_ i, r, g;

	v = rand() % 7;
	d = Go[v][0];
	z = 0;
	j = (10 - 4) / 2;
	k = (v + 1) / -4;

	n(i,8)
		m[i] = v * 8 + i;

	n(i,7)
		r = rand() % (8 - i),
		g = m[7 - i],
		m[7 - i] = m[r],
		m[r] = g;

	o H(d, j, k);
}



/* Gl:
*/
a Gl(u bmp, _ x1, _ y1, _ x2, _ y2)
{
	_ x, y;
	for(y = y1; y < y2; y += 512)
		for(x = x1; x < x2; x += 512)
			blit(q, bmp, 0, 0, x, y, 512, 512);
}



/* Gm:
*/
Gm()
{
	_ b, r, i = 100, r_ = 100, Z,
		 rb, x, y, ra, rc,
		 rd, re, rf, rg, rh, ri,
		 rj, rk, rl, g = 0, rm = -1,
		 rn, rz, ry;

	t = 0;

	n(y,20+1)
		n(x,10+1)
			l[y][x].x = l[y][x].d =
										(((x+1) * e) << 16) + 0x8000,
			l[y][x].y = l[y][x].b =
										(((y+1) * f) << 16) + 0x8000;

	for(y = 80; y < 180; y++)
		for(x = h/3; x < h*2/3; x++)
			putpixel(screen, x, y, 184 + getpixel(screen, x, y) * 19 / 64);

	textprintf(screen, font, 8, E - 16, 255, "Hiscore: %d", B);

	memset(c, 0, sizeof(c));
	rf = rd = re = rg = rj = s;
	A = 0;
	p(1) {
		if(!Gk())
			t = 1, rn = s;
		b = s;
		r = 1;
		rl = 0;

		/* Drop one piece */
		p(!rl) {
			p(rj < s)
				Gh(),
				rj += 50,
				r = 1;

			if(!t) {
				do {
					rk = 0;
					if(s > b + 40 + 4 * i) {
						/* Move down. */
						b += 40 + 4 * i;
						rk = 1;
						if(H(d, j, k + 1))
							k++;
						else {
							/* Freeze it. */
							g += I(d, j, k);
							rl = 1;
							break;
						}
						r = 1;
					}
					else {
						if(key[KEY_LEFT] && !key[KEY_RIGHT]) {
							if(s > rd + 100) {
								if(H(d, j - 1, k)) {
									rd ?
										(rd += 100,
										rk = 1)
									:
										(rd = s);
									j--;
									r = 1;
								}
							}
						}
						else
							rd = 0;

						if(key[KEY_RIGHT] && !key[KEY_LEFT]) {
							if(re + 100 < s) {
								if(H(d, j + 1, k)) {
									re ?
										(re += 100,
										rk = 1)
									:
										(re = s);
									j++;
									r = 1;
								}
							}
						}
						else
							re = 0;

						if(key[KEY_SPACE] | key[KEY_DOWN]) {
							if(rf + 500 < s) {
								rf = s;
								if(H(d, j, k + 1)) {
									k++;
									b = s;
									p(H(d, j, k+1))
										k++;
								}
								else {
									g += I(d, j, k);
									rl = 1;
									break;
								}
								g++;
								r = 1;
							}
						}
						else
							rf = 0;

						if(key[KEY_UP]) {
							if(rg + 200 < s) {
								Z = Go[v][(z+1)&3];
								rb = 0;
								rh = 1;
								ri = 1;
								if(((Z & 0x1111) && !(Z & 0x8888))|
									((Z & 0x2222) && !(Z & 0x4444)))
									ra = 2;
								else if(((d & 0x1111) && !(d & 0x8888)) |
										  ((d & 0x2222) && !(d & 0x4444)))
									ra = -2, rh = -1;
								else
									ra = 1;

								if(((Z & 0xf000) && !(Z & 0x000f))|
									((Z & 0x0f00) && !(Z & 0x00f0)))
									rc = 2;
								else if(((d & 0xf000) && !(d & 0x000f)) |
									((d & 0x0f00) && !(d & 0x00f0)))
									rc = -2, ri = -1;
								else
									rc = 1;
								for(y = 0; y != rc && !rb; y += ri)
									for(x = 0; x != ra; x += rh)
										if(H(Z,
															 j + x, k + y)) {
											z++;
											rg ?
												rg += 100,
												rk = 1
											:
												(rg = s);
											d = Z;
											j += x;
											k += y;
											r = 1;
											rb = 1;
											break;
										}
							}
						}
						else
							rg = 0;
					}
				} p(rk);
			}
			else if(s < rn + 500)
				clear_keybuf();
			else if(keypressed())
				o g;

			if(key[KEY_ESC])
				t = 1, rn = s;

			if(key[KEY_P] && !t && (ry + 500 < s)) {
				rz = s;
				rectfill(screen, h, C,
									  h + e*10,
									  C + f*20, 0);
				p(key[KEY_P])
					; /* do nothing */
				p(!key[KEY_P])
					; /* do nothing */
				r = 1;
				rk = 0;
				b += s - rz;
				rj += s - rz;
				ry = s;
			}

			r ?
				Gj(Gp, F),
				blit(F, screen, 0, 0, h, C,
					  e*10, f * 20),
				r = 0 : 0;

			if(rm != g) {
				blit(score_bmp, screen, 0, 0, 8, 8, score_bmp->w, score_bmp->h);
				textprintf(screen, font, 8*9, 16, 255, "%d", g);
				textprintf(screen, font, 8*9, 32, 255, "%d", A);
				rm = g;
				i = 100 - A;
				if(i < 0)
					i = 0;
				for(y = 80 + i; y < 80 + r_; y++)
					for(x = h / 3; x < h * 2 / 3; x++)
						putpixel(screen, x, y, getpixel(screen, x, y) + 51);
				r_ = i;
			}
		}
	}
}



#define QUIT(msg) { allegro_exit(); allegro_message(msg); o 1; }



/* Gn:
*/
Gn()
{
	_ rz = 1, Z = 1, r, g, b, i, x, y, r_, ra,
		 rb;
	u rc, *rd;

	Gf(q);

	rc = create_bitmap(8 * 13, 40);
	rd = create_bitmap(D / 2 + 2, E / 2 + 2);
	if((rc == NULL) | (rd == NULL)
		J
	clear(rc);
	Gl(rd, -D / 4, -E / 4,
								 D * 3 / 4, E * 3 / 4);
	textout(rc, font, "SPACE - START",       0,  0, 255);
	textout(rc, font, "    A - ABOUT",       0, 16, 255);
	textout(rc, font, "  ESC - QUIT",        0, 32, 255);
	stretch_sprite(rd, rc, 1, 1, D / 2, E / 2);
	r_ = rd->w - 1;
	ra = rd->h - 1;
	for(y = 1; y < ra; y++) {
		for(x = 1; x < r_; x++) {
			if(rd->line[y][x] == 255) {
				(!(rd->line[y-1][x] & 128) ||
					!(rd->line[y][x-1] & 128) ||
					!(rd->line[y+1][x] & 128) ||
					!(rd->line[y][x+1] & 128)) ?
					rd->line[y][x] = 254
				:
					(rd->line[y][x] = 204 + abs(30 - (x*30)/rd->w -
																 (y*30)/rd->h));
			}
		}
	}

	n(y,40)
		n(x,120)
			score_bmp->line[y][x] = q->line[y][x] * 3 / 4;
	text_mode(-1);
	textout(score_bmp, font, "Score: ", 8, 8, 255);
	textout(score_bmp, font, "Lines: ", 8, 24, 255);

	r = 31 + (rand() % 32);
	g = 31 + (rand() % 32);
	b = 31 + (rand() % 32);
	#define FILL(x, y, r1, g1, b1, r2, g2, b2)                              \
	n(i,y)                                                                  \
		Ga[x+i].r = r1 + (i * r2) / (y - 1),                            \
		Ga[x+i].g = g1 + (i * g2) / (y - 1),                            \
		Ga[x+i].b = b1 + (i * b2) / (y - 1);

	FILL(0, 64, 0, 0, 0, r, g, b);
	FILL(64, 20, 0, 0, 0, 63, 0, 63);
	FILL(84, 20, 0, 0, 0, 63, 63, 0);
	FILL(104, 20, 0, 0, 0, 63, 0, 0);
	FILL(124, 20, 0, 0, 0, 63, 31, 0);
	FILL(144, 20, 0, 0, 0, 0, 63, 0);
	FILL(164, 20, 0, 0, 0, 0, 63, 63);
	FILL(184, 20, 0, 0, 0, 0, 0, 63);
	FILL(204, 31, 63, 0, 0, 0, 63, 0);
	FILL(235, 19, 63, 15, 0, 0, 48, 0);
	FILL(254, 2, 0, 0, 0, 63, 63, 63);
	set_palette(Ga);

	do {
		Z ?
			Gl(screen, 0, 0, D, E),
			blit(rd, screen, 0, 0, D / 4, E / 4,
									 D / 2, E / 2),
			Z = 0 : 0;

		p(!keypressed())
			; /* do nothing */
		switch(readkey() >> 8) {
			case KEY_SPACE:
				Gl(screen, 0, 0, D, E);
				rb = Gm();
				if(rb > B)
					B = rb;
				clear_keybuf();
				Z = 1;
				break;

			case KEY_A:
				Gl(screen, 0, 0, D, E);
				#define PRINT(text, y) textprintf_centre(screen, font, D / 2, E / 2 + y, 255, text)
				PRINT("Tetris", -52);
				PRINT("By Sven Sandberg", -36);
				PRINT("April 2000", -28);
				PRINT("Thanks to", -4);
				PRINT("Shawn Hargreaves", 12);
				PRINT("Creator of Allegro, the best game programming library in the world.", 20);
				PRINT("DJ Delorie & friends", 36);
				PRINT("Creator of DJGPP", 44);
				clear_keybuf();
				readkey();
				Z = 1;
				break;

			case KEY_ESC:
				rz = 0;
				break;
		}
	} p(rz);


	fade_out(32);

	o 0;
}



/* main:
*/
main(_ argc, char *argv[])
{
	u r;
	_ i;
	PACKFILE *b;

	/* Install Allegro. */
	allegro_init();
	if(set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0))
		QUIT("Unable to set graphics mode");
	install_timer();
	install_keyboard();

	s = 0;
	LOCK_VARIABLE(s);
	LOCK_FUNCTION(Gc);
	install_int(Gc, 10);

	/* Set up some variables. */
	if((argc < 2) || (strcmp(argv[1], "--no-rand") != 0))
		srand(time(NULL));
	f = E / 20;
	e = (3 * D * f) / (4 * E);
	C = (E - f * 20) / 2;
	h = (D - e * 10) / 2;

	/* Create bitmaps. */
	r = create_bitmap(e, f);
	if(r == NULL
		J
	n(i,56) {
		G[i] = create_bitmap(e, f);
		if(G[i] == NULL
			J
		Gg(G[i], i/8, r);
	}
	destroy_bitmap(r);

	score_bmp = create_bitmap(8*15, 8*5);
	F = create_bitmap(10 * e, 20 * f);
	Gp = create_bitmap((10 + 2) * e,
											 (20 + 2) * f);
	q = create_bitmap(512, 512);
	if((q == NULL) | (F == NULL) |
		(Gp == NULL) | (score_bmp == NULL)
		J
	clear(Gp);

	/* Load B */
	packfile_password("I'm the password for the datafile");
	b = pack_fopen("hiscore.dat", F_READ_PACKED);
	if(b != NULL) {
		if((B = pack_igetl(b)) == EOF)
			B = 0;
		pack_fclose(b);
	}

	/* Play. */
	if(Gn())
		o 1;

	/* Destroy bitmaps. */
	n(i,56)
		destroy_bitmap(G[i]);
	destroy_bitmap(q);
	destroy_bitmap(F);
	destroy_bitmap(Gp);
	destroy_bitmap(score_bmp);

	/* Save hiscore */
	b = pack_fopen("hiscore.dat", F_WRITE_PACKED);
	if(b == NULL)
		QUIT("Error opening hiscore file");
	if(pack_iputl(B, b) == EOF)
		QUIT("Error writing to hiscore file");
	pack_fclose(b);

	o 0;
}
END_OF_MAIN();

